@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 function anjuta_tags {
 
@@ -6,7 +6,7 @@ function anjuta_tags {
     BASEDIR=`pwd`
     PROGDIR=. # `dirname $0`
     CFLAGS=""
-    ANJUTA_TAGS_BIN="`which anjuta-tags`"
+    ANJUTA_TAGS_BIN="/usr/local/share/anjuta/scripts/anjuta-tags"
     OUTPUT_FILE=$1
     shift
     
@@ -38,7 +38,7 @@ function anjuta_tags {
     
     if [ $FILES_COUNT -gt 0 ] ; then
 	rm -f $OUTPUT_FILE
-	$ANJUTA_TAGS_BIN $OUTPUT_FILE $FILES 2>log.txt
+	$ANJUTA_TAGS_BIN $OUTPUT_FILE $FILES >"$OUTPUT_FILE.log" 2>&1
 	
     # If global tags file could not be generated
 	if [ ! -e $OUTPUT_FILE ] ; then
@@ -54,37 +54,49 @@ FILES=""
 BASEDIR=`pwd`
 PROGDIR=. # `dirname $0`
 CFLAGS=""
-ANJUTA_TAGS="anjuta_tags"
 
+echo "Generating System tags..."
 
-echo "[...........................................................................]"
-echo "[. Generating System tags. This may take several minutes (1 min to 20 mins}.]"
-echo "[. You can go out, have a coffee and return back in 20 mins.]...............]"
-echo "[...........................................................................]"
-
-mkdir -p "$BASEDIR/tags"
+mkdir -p "$HOME/.anjuta/tags"
 
 # pkg-config libraries
 PKG_CONFIG=`which pkg-config 2>/dev/null`
 if ( [ ! -z $PKG_CONFIG ] && [ -x $PKG_CONFIG ] ) ; then
-    
+    ##
+	## We are scaning .pc files directly because --cflags option
+	## produces include paths and flags for other packages depended by
+	## the packages, which we don't like. We only want cflags relevent
+	## to the packages alone.
+	##
     PKG_DEFAULT_CONFIG_PATH=`echo $PKG_CONFIG | sed -e 's/\/bin\/pkg-config/\/lib\/pkgconfig'/`
     PKG_CONFIG_PATH="$PKG_CONFIG_PATH:$PKG_DEFAULT_CONFIG_PATH"
     PKG_DEFAULT_CONFIG_PATH=`echo $PKG_CONFIG | sed -e 's/\/bin\/pkg-config/\/lib64\/pkgconfig'/`
     PKG_CONFIG_PATH="$PKG_CONFIG_PATH:$PKG_DEFAULT_CONFIG_PATH"
 	
     PKG_CONFIG_PATH_PROPER=`echo $PKG_CONFIG_PATH | sed -e 's/\:/ /g'`
+
+    pkg_files_count=0
     for pkg_path in $PKG_CONFIG_PATH_PROPER; do
-	for pkg_file in $pkg_path/*.pc; do
+	for pkg_file in `find "$pkg_path" -name '*.pc' 2>/dev/null`; do
+	    pkg_files_count=`expr $pkg_files_count + 1`
+	done
+    done
+
+    ## This line is important for the progress bar. Don't change it.
+    echo "Scanning $pkg_files_count packages"
+
+    for pkg_path in $PKG_CONFIG_PATH_PROPER; do
+	for pkg_file in `find "$pkg_path" -name '*.pc' 2>/dev/null`; do
 		# echo "Scanning package: $pkg_file"
-	    PKG_CFLAGS=`cat $pkg_file  2>/dev/null | grep -e "^\(\w\+\=\|\s*Cflags\:\)" | sed -e "s/^\s*Cflags\:\(.*\)/echo \"\1\"/" | sed -e 's/^\(\w\+\)\=\(.*\)/\1\=\"\2\"/' | sh`
+	    PKG_CFLAGS=`grep -e '^\(\w\+=\|\s*Cflags:\)' "$pkg_file"|sed -e 's/Cflags:\s*"\?\(.*\)"\?\s*/echo "\1"/' -e 's/^\(\w\+\)=\("\([^"]*\)"\|\(.*\)\)/\1="\3\4"/'|sh`
 		# echo $PKG_CFLAGS
-	    package=`echo $pkg_file | sed -e 's/^.*\/\(.*\)\.pc/\1/'`
+	    package=`basename "$pkg_file" '.pc'`
 	    # if [ ! -e $BASEDIR/tags/$package.anjutatags.gz ] ; then
 		echo "Creating tags file: $HOME/.anjuta/tags/$package.anjutatags.gz ..."
 		anjuta_tags "$HOME/.anjuta/tags/$package.anjutatags" $PKG_CFLAGS > /dev/null
 	    # fi
 	done
     done
+	echo "Done"
     echo "This is just a timestamp" > $HOME/.anjuta/system-tags.cache
 fi
