@@ -1,6 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
-/* anjuta-launcher.c
- * Copyright (C) 2003 Naba Kumar
+/*
+ * anjuta-launcher.c
+ * Copyright (C) 2003 Naba Kumar  <naba@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,6 +17,16 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
+/**
+ * SECTION:anjuta-launcher
+ * @short_description: External process launcher with async input/output
+ * @see_also: 
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-launcher.h
+ * 
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -24,9 +35,16 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
-#ifndef sun
-#include <pty.h>
+
+#if !defined(__sun) && !defined(__NetBSD__)
+#  ifndef FREEBSD
+#    include <pty.h>
+#  else
+#    include <libutil.h>
+#  endif
 #endif
+#include "anjuta-utils-priv.h"
+
 #include <assert.h>
 #include <termios.h>
 
@@ -37,7 +55,6 @@
 
 #include <libgnome/gnome-macros.h>
 
-
 #include "anjuta-children.h"
 #include "anjuta-utils.h"
 #include "anjuta-marshal.h"
@@ -48,8 +65,12 @@
 #define ANJUTA_PIXMAP_PASSWORD "password.png"
 #define FILE_BUFFER_SIZE 1024
 #ifndef __MAX_BAUD
+#ifdef __CYGWIN__
+#define __MAX_BAUD B256000
+#else
 #define __MAX_BAUD B460800
 #endif
+#endif
 
 /*
 static gboolean
@@ -86,7 +107,7 @@ struct _AnjutaLauncherPriv
 	/* Output of the pty is constantly stored here.*/
 	gchar *pty_output_buffer;
 
-	/* Terminal ehco */
+	/* Terminal echo */
 	gboolean terminal_echo_on;
 	
 	/* The child */
@@ -97,7 +118,10 @@ struct _AnjutaLauncherPriv
 	/* Synchronization in progress */
 	gboolean in_synchronization;
 	guint completion_check_timeout;
-	
+
+	/* Terminate child on child exit */
+	gboolean terminate_on_exit;
+
 	/* Start time of execution */
 	time_t start_time;
 	
@@ -124,6 +148,7 @@ enum
 
 static void anjuta_launcher_class_init (AnjutaLauncherClass * klass);
 static void anjuta_launcher_init (AnjutaLauncher * obj);
+static gboolean anjuta_launcher_call_execution_done (gpointer data);
 static gboolean anjuta_launcher_check_for_execution_done (gpointer data);
 static void anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 													gboolean emit_signal);
@@ -166,6 +191,9 @@ anjuta_launcher_initialize (AnjutaLauncher *obj)
 	obj->priv->in_synchronization = FALSE;
 	obj->priv->completion_check_timeout = -1;
 	
+	/* Terminate child on child exit */
+	obj->priv->terminate_on_exit = FALSE;
+
 	/* Start time of execution */
 	obj->priv->start_time = 0;
 	
@@ -240,23 +268,46 @@ anjuta_launcher_class_init (AnjutaLauncherClass * klass)
 	/* DEBUG_PRINT ("Initializing launcher class"); */
 	
 	parent_class = g_type_class_peek_parent (klass);
+
+	/**
+	 * AnjutaLauncher::child-exited
+ 	 * @launcher: a #AnjutaLancher object.
+	 * @child_pid: process ID of the child
+	 * @status: status as returned by waitpid function
+	 * @time: time in seconds taken by the child
+	 * 
+	 * Emitted when the child has exited and all i/o channels have
+	 * been closed. If the terminate on exit flag is set, the i/o
+	 * channels are automatically closed when the child exit.
+	 * You need to use WEXITSTATUS and friend to get the child exit
+	 * code from the status returned.
+	 **/
 	launcher_signals[CHILD_EXITED_SIGNAL] =
 		g_signal_new ("child-exited",
 					G_TYPE_FROM_CLASS (object_class),
 					G_SIGNAL_RUN_FIRST,
 					G_STRUCT_OFFSET (AnjutaLauncherClass,
-									 child_exited_signal),
+									 child_exited),
 					NULL, NULL,
 					anjuta_cclosure_marshal_VOID__INT_INT_ULONG,
 					G_TYPE_NONE, 3, G_TYPE_INT,
 					G_TYPE_INT, G_TYPE_ULONG);
 	
+	/**
+	 * AnjutaLauncher::busy
+ 	 * @launcher: a #AnjutaLancher object.
+	 * @busy: TRUE is a child is currently running
+	 * 
+	 * Emitted when a child starts after a call to one execute function
+	 * (busy is TRUE) or when a child exits and all i/o channels are
+	 * closed (busy is FALSE).
+	 **/
 	launcher_signals[BUSY_SIGNAL] =
 		g_signal_new ("busy",
 					G_TYPE_FROM_CLASS (object_class),
 					G_SIGNAL_RUN_FIRST,
 					G_STRUCT_OFFSET (AnjutaLauncherClass,
-									 busy_signal),
+									 busy),
 					NULL, NULL,
 					anjuta_cclosure_marshal_VOID__BOOLEAN,
 					G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
@@ -401,7 +452,7 @@ anjuta_launcher_synchronize (AnjutaLauncher *launcher)
 	{
 		launcher->priv->in_synchronization = TRUE;
 		
-		if (launcher->priv->completion_check_timeout)
+		if (launcher->priv->completion_check_timeout >= 0)
 			g_source_remove (launcher->priv->completion_check_timeout);
 		launcher->priv->completion_check_timeout = 
 		    g_timeout_add (50, anjuta_launcher_check_for_execution_done,
@@ -414,10 +465,23 @@ anjuta_launcher_synchronize (AnjutaLauncher *launcher)
 			 launcher->priv->stderr_is_done)
 	{
 		/* DEBUG_PRINT ("Child has't exited yet waiting for 200ms"); */
+		if (launcher->priv->completion_check_timeout >= 0)
+			g_source_remove (launcher->priv->completion_check_timeout);
 		launcher->priv->completion_check_timeout = 
 		    g_timeout_add(200, anjuta_launcher_check_for_execution_done,
 							launcher);
 	}
+	/* Add this case for gdb. It creates child inheriting gdb
+	 * pipes which are not closed if gdb crashes */
+	else if (launcher->priv->child_has_terminated &&
+			launcher->priv->terminate_on_exit)
+	{
+		if (launcher->priv->completion_check_timeout >= 0)
+			g_source_remove (launcher->priv->completion_check_timeout);
+		launcher->priv->completion_check_timeout = 
+		    g_timeout_add(0, anjuta_launcher_call_execution_done,
+							launcher);
+	}
 }
 
 /* Password dialog */
@@ -798,28 +862,45 @@ static void
 anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 										gboolean emit_signal)
 {
+	static gboolean block = FALSE;
 	gint child_status, child_pid;
 	time_t start_time;
 	
 	if (launcher->priv->completion_check_timeout >= 0)
 		g_source_remove (launcher->priv->completion_check_timeout);
 	
+	if (block)
+		return;
+	
 	/* Make sure all pending I/O are flushed out */
+	block = TRUE;
 	while (g_main_context_pending (NULL))
 		g_main_context_iteration (NULL, FALSE);
 	
-	g_io_channel_shutdown (launcher->priv->stdout_channel, emit_signal, NULL);
-	g_io_channel_shutdown (launcher->priv->stderr_channel, emit_signal, NULL);
-	g_io_channel_shutdown (launcher->priv->pty_channel, emit_signal, NULL);
-	
-	g_io_channel_unref (launcher->priv->stdout_channel);
-	g_io_channel_unref (launcher->priv->stderr_channel);
-	g_io_channel_unref (launcher->priv->pty_channel);
-	
-	g_source_remove (launcher->priv->pty_watch);
-	g_source_remove (launcher->priv->stdout_watch);
-	g_source_remove (launcher->priv->stderr_watch);
+	/* Can be called again, while waiting in the previous line
+	 * Do nothing if clean up is already done */
+	if (launcher->priv->stdout_channel)
+	{	
+		g_io_channel_shutdown (launcher->priv->stdout_channel, emit_signal, NULL);
+		g_io_channel_unref (launcher->priv->stdout_channel);
+		g_source_remove (launcher->priv->stdout_watch);
+	}
+
+	if (launcher->priv->stderr_channel)
+	{
+		g_io_channel_shutdown (launcher->priv->stderr_channel, emit_signal, NULL);
+		g_io_channel_unref (launcher->priv->stderr_channel);
+		g_source_remove (launcher->priv->stderr_watch);
+	}
+
+	if (launcher->priv->pty_channel)
+	{
+		g_io_channel_shutdown (launcher->priv->pty_channel, emit_signal, NULL);
+		g_io_channel_unref (launcher->priv->pty_channel);
 	
+		g_source_remove (launcher->priv->pty_watch);
+	}
+
 	if (launcher->priv->pty_output_buffer)
 		g_free (launcher->priv->pty_output_buffer);
 	if (launcher->priv->stdout_buffer)
@@ -828,7 +909,7 @@ anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 		g_free (launcher->priv->stdout_buffer);
 	
 	/* Save them before we re-initialize */
-	child_status = WEXITSTATUS(launcher->priv->child_status);
+	child_status = launcher->priv->child_status;
 	child_pid = launcher->priv->child_pid;
 	start_time = launcher->priv->start_time;
 	
@@ -846,6 +927,21 @@ anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 		g_signal_emit_by_name (launcher, "child-exited", child_pid,
 							   child_status,
 							   time (NULL) - start_time);
+		
+	block = FALSE;
+}
+
+/* Using this function is necessary because
+ * anjuta_launcher_execution_done_cleanup needs to be called in the same
+ * thread than the gtk main loop */
+static gboolean
+anjuta_launcher_call_execution_done (gpointer data)
+{
+	AnjutaLauncher *launcher = data;
+
+	launcher->priv->completion_check_timeout = -1;
+	anjuta_launcher_execution_done_cleanup (launcher, TRUE);
+	return FALSE;
 }
 
 /* monitors closure of stdout stderr and pty through a gtk_timeout_add setup */
@@ -876,6 +972,8 @@ anjuta_launcher_child_terminated (int status, gpointer data)
 {
 	AnjutaLauncher *launcher = data;
 	
+	g_return_if_fail(ANJUTA_IS_LAUNCHER(launcher));
+	
 	/* Save child exit code */
 	launcher->priv->child_status = status;
 	launcher->priv->child_has_terminated = TRUE;
@@ -956,6 +1054,7 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 		
 		execvp (args[0], args);
 		g_warning (_("Cannot execute command: \"%s\""), args[0]);
+		perror(_("execvp failed"));
 		_exit(-1);
 	}
 	g_free (working_dir);
@@ -1004,7 +1103,10 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 
 	if (!launcher->priv->terminal_echo_on)
 	{
-		termios_flags.c_lflag &= ~(ECHOKE | ECHOE | ECHO | ECHONL | ECHOPRT |
+		termios_flags.c_lflag &= ~(ECHOKE | ECHOE | ECHO | ECHONL | 
+#ifdef ECHOPRT
+						ECHOPRT |
+#endif
 						ECHOCTL | ISIG | ICANON | IEXTEN | NOFLSH | TOSTOP);
 	}
 //	termios_flags.c_lflag |= 0;
@@ -1118,14 +1220,49 @@ anjuta_launcher_execute (AnjutaLauncher *launcher, const gchar *command_str,
 }
 
 /**
- * anjuta_launcher_echo:
+ * anjuta_launcher_set_buffered_output:
+ * @launcher: a #AnjutaLancher object.
+ * @buffered: buffer output.
+ * 
+ * Sets if output should buffered or not. By default, it is buffered.
+ *
+ * Return value: Previous flag value
+ */
+gboolean
+anjuta_launcher_set_buffered_output (AnjutaLauncher *launcher, gboolean buffered)
+{
+	gboolean past_value = launcher->priv->buffered_output;
+	launcher->priv->buffered_output = buffered;
+	return past_value;
+}
+
+/**
+ * anjuta_launcher_set_check_passwd_prompt:
+ * @launcher: a #AnjutaLancher object.
+ * @check_passwd: check for password.
+ * 
+ * Set if output is checked for a password prompti. A special dialog box
+ * is use to enter it in this case. By default, this behavior is enabled.
+ *
+ * Return value: Previous flag value
+ */
+gboolean
+anjuta_launcher_set_check_passwd_prompt (AnjutaLauncher *launcher, gboolean check_passwd)
+{
+	gboolean past_value = launcher->priv->check_for_passwd_prompt;
+	launcher->priv->check_for_passwd_prompt = check_passwd;
+	return past_value;
+}
+
+/**
+ * anjuta_launcher_set_terminal_echo:
  * @launcher: a #AnjutaLancher object.
  * @echo_on: Echo ON flag.
  * 
  * Sets if input (those given in STDIN) should enabled or disabled. By default,
  * it is disabled.
  *
- * Return value: TRUE if successfully set, otherwise FALSE.
+ * Return value: Previous flag value
  */
 gboolean
 anjuta_launcher_set_terminal_echo (AnjutaLauncher *launcher,
@@ -1136,6 +1273,27 @@ anjuta_launcher_set_terminal_echo (AnjutaLauncher *launcher,
 	return past_value;
 }
 
+/**
+ * anjuta_launcher_set_terminate_on_exit:
+ * @launcher: a #AnjutaLancher object.
+ * @terminate_on_exit: terminate on exit flag
+ * 
+ * When this flag is set, al i/o channels are closed and the child-exit
+ * signal is emitted as soon as the child exit. By default, or when this
+ * flag is clear, the launcher object wait until the i/o channels are
+ * closed.
+ *
+ * Return value: Previous flag value
+ */
+gboolean
+anjuta_launcher_set_terminate_on_exit (AnjutaLauncher *launcher,
+						gboolean terminate_on_exit)
+{
+	gboolean past_value = launcher->priv->terminate_on_exit;
+	launcher->priv->terminate_on_exit = terminate_on_exit;
+	return past_value;
+}
+
 /**
  * anjuta_launcher_new:
  * 
