@@ -27,6 +27,7 @@
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <libanjuta/interfaces/ianjuta-file-loader.h>
 #include <libanjuta/interfaces/ianjuta-vcs.h>
+#include <libanjuta/interfaces/ianjuta-document-manager.h>
 
 #include <plugins/project-wizard/autogen.h>
 
@@ -221,6 +222,7 @@ cg_plugin_add_to_project (AnjutaClassGenPlugin *plugin,
 	{
 		*new_header_file = added_files->data;
 		*new_source_file = g_list_next (added_files)->data;
+
 		result = TRUE;
 	}
 
@@ -264,6 +266,43 @@ cg_plugin_generator_error_cb (G_GNUC_UNUSED CgGenerator *generator,
 		GTK_WIDGET (cg_window_get_dialog (plugin->window)), TRUE);
 }
 
+static gboolean
+cg_plugin_load (AnjutaClassGenPlugin *plugin,
+                CgGenerator *generator,
+                const gchar *file,
+                GError **error)
+{
+	IAnjutaDocumentManager *docman;
+	IAnjutaEditor *editor;
+	gchar *name;
+	gchar *contents;
+	gboolean result;
+
+	docman = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
+	                                     IAnjutaDocumentManager, NULL);	
+
+	if(g_file_get_contents(file, &contents, NULL, error) == FALSE)
+		return FALSE;
+
+	name = g_path_get_basename (file);
+
+	result = FALSE;
+	/* The content argument seems not to work */
+	editor = ianjuta_document_manager_add_buffer (docman, name, "", error);
+
+	if(editor != NULL)
+	{
+		ianjuta_editor_append(editor, contents, -1, error);
+		if(!error || *error == NULL)
+			result = TRUE;
+	}
+	
+	g_free(contents);
+	g_free(name);
+	
+	return result;
+}
+
 static void
 cg_plugin_generator_created_cb (CgGenerator *generator,
                                 gpointer user_data)
@@ -271,10 +310,7 @@ cg_plugin_generator_created_cb (CgGenerator *generator,
 	AnjutaClassGenPlugin *plugin;
 	const gchar *header_file;
 	const gchar *source_file;
-	gchar *new_header_file;
-	gchar *new_source_file;
 	IAnjutaFileLoader *loader;
-	gboolean result;
 
 	plugin = (AnjutaClassGenPlugin *) user_data;
 	header_file = cg_generator_get_header_destination (generator);
@@ -283,41 +319,27 @@ cg_plugin_generator_created_cb (CgGenerator *generator,
 	loader = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
 	                                     IAnjutaFileLoader, NULL);
 
+	if (cg_window_get_add_to_repository (plugin->window))
+	{
+		cg_plugin_add_to_repository (plugin, header_file, source_file);
+	}
+
 	if (cg_window_get_add_to_project (plugin->window))
 	{
-		result = cg_plugin_add_to_project (plugin, header_file, source_file,
-		                                   &new_header_file, &new_source_file);
+		ianjuta_file_loader_load (loader, header_file, FALSE, NULL);
+		ianjuta_file_loader_load (loader, source_file, FALSE, NULL);
 	}
 	else
 	{
-		new_header_file = g_strdup (header_file);
-		new_source_file = g_strdup (source_file);
-
-		result = TRUE;
+		/* We do not just use ianjuta_file_leader_load here to ensure that
+		 * the new documents are flagged as changed and no path is
+		 * already set. */
+		cg_plugin_load (plugin, generator, header_file, NULL);
+		cg_plugin_load (plugin, generator, source_file, NULL);
 	}
 
-	if (result == TRUE)
-	{
-		if (cg_window_get_add_to_repository (plugin->window))
-		{
-			cg_plugin_add_to_repository (plugin, new_header_file,
-			                             new_source_file);
-		}
-
-		ianjuta_file_loader_load (loader, new_header_file, FALSE, NULL);
-		ianjuta_file_loader_load (loader, new_source_file, FALSE, NULL);
-
-		g_free (new_header_file);
-		g_free (new_source_file);
-
-		g_object_unref (G_OBJECT (plugin->window));
-		plugin->window = NULL;
-	}
-	else
-	{
-		gtk_widget_set_sensitive (
-			GTK_WIDGET (cg_window_get_dialog (plugin->window)), TRUE);
-	}
+	g_object_unref (G_OBJECT (plugin->window));
+	plugin->window = NULL;
 }
 
 static void
@@ -332,63 +354,89 @@ cg_plugin_window_response_cb (G_GNUC_UNUSED GtkDialog *dialog,
 	GError *error;
 	gchar *name;
 
+	gchar *header_file;
+	gchar *source_file;
+	gboolean result;
+
 	plugin = (AnjutaClassGenPlugin *) user_data;
 	error = NULL;
 
 	if (response_id == GTK_RESPONSE_ACCEPT)
 	{
-		values = cg_window_create_value_heap (plugin->window);
-
-		manager = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
-		                                      IAnjutaProjectManager, NULL);
-
-		if (manager != NULL && plugin->top_dir != NULL)
-		{
-			/* Use basename of the project's root URI as project name. */
-			name = g_path_get_basename (plugin->top_dir);
-			value = npw_value_heap_find_value (values, "ProjectName");
-			npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
-			g_free (name);
+    	if (cg_window_get_add_to_project (plugin->window))
+	    {
+		    result = cg_plugin_add_to_project (
+		    	plugin, cg_window_get_header_file (plugin->window),
+				cg_window_get_source_file (plugin->window),
+				&header_file, &source_file);
 		}
 		else
 		{
-			name = g_path_get_basename (cg_window_get_source_file(
-			                            plugin->window));
-			value = npw_value_heap_find_value (values, "ProjectName");
-			npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
-			g_free (name);
+		    header_file = g_build_filename (g_get_tmp_dir (),
+		    	cg_window_get_header_file (plugin->window), NULL);
+		    source_file = g_build_filename (g_get_tmp_dir (),
+		    	cg_window_get_source_file (plugin->window), NULL);
+
+		    result = TRUE;
 		}
-
-		plugin->generator = cg_generator_new (
-			cg_window_get_header_template(plugin->window),
-			cg_window_get_source_template(plugin->window),
-			cg_window_get_header_file(plugin->window),
-			cg_window_get_source_file(plugin->window));
-
-		if (cg_generator_run(plugin->generator, values, &error) == FALSE)
+    	
+		if(result == TRUE)
 		{
-			anjuta_util_dialog_error (
-				GTK_WINDOW (cg_window_get_dialog (plugin->window)),
-				_("Failed to execute autogen: %s"), error->message);
+			values = cg_window_create_value_heap (plugin->window);
 
-			g_object_unref (G_OBJECT (plugin->generator));
-			g_error_free (error);
+			manager = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
+			                                      IAnjutaProjectManager, NULL);
+
+			if (manager != NULL && plugin->top_dir != NULL)
+			{
+				/* Use basename of the project's root URI as project name. */
+				name = g_path_get_basename (plugin->top_dir);
+				value = npw_value_heap_find_value (values, "ProjectName");
+				npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
+				g_free (name);
+			}
+			else
+			{
+				name = g_path_get_basename (cg_window_get_source_file(
+				                            plugin->window));
+				value = npw_value_heap_find_value (values, "ProjectName");
+				npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
+				g_free (name);
+			}
+
+    		plugin->generator = cg_generator_new (
+				cg_window_get_header_template(plugin->window),
+				cg_window_get_source_template(plugin->window),
+				header_file,
+				source_file);
+
+			if (cg_generator_run (plugin->generator, values, &error) == FALSE)
+			{
+				anjuta_util_dialog_error (
+					GTK_WINDOW (cg_window_get_dialog (plugin->window)),
+					_("Failed to execute autogen: %s"), error->message);
+
+				g_object_unref (G_OBJECT (plugin->generator));
+				g_error_free (error);
+			}
+			else
+			{
+				g_signal_connect (G_OBJECT (plugin->generator), "error",
+				                  G_CALLBACK (cg_plugin_generator_error_cb),
+				                  plugin);
+
+				g_signal_connect (G_OBJECT (plugin->generator), "created",
+				                 G_CALLBACK (cg_plugin_generator_created_cb),
+								 plugin);
+
+				gtk_widget_set_sensitive (
+					GTK_WIDGET (cg_window_get_dialog (plugin->window)), FALSE);
+			}
+
+			npw_value_heap_free (values);
+			g_free (header_file);
+			g_free (source_file);
 		}
-		else
-		{
-			g_signal_connect (G_OBJECT (plugin->generator), "error",
-			                  G_CALLBACK (cg_plugin_generator_error_cb),
-			                  plugin);
-
-			g_signal_connect (G_OBJECT (plugin->generator), "created",
-			                 G_CALLBACK (cg_plugin_generator_created_cb),
-							 plugin);
-
-			gtk_widget_set_sensitive (
-				GTK_WIDGET (cg_window_get_dialog (plugin->window)), FALSE);
-		}
-
-		npw_value_heap_free (values);
 	}
 	else
 	{
@@ -417,8 +465,11 @@ iwizard_activate (IAnjutaWizard *wiz, G_GNUC_UNUSED GError **err)
 	user_email = anjuta_preferences_get (cg_plugin->prefs,
 	                                     "anjuta.user.email");
 	
-	cg_window_set_author (cg_plugin->window, user_name);
-	cg_window_set_email (cg_plugin->window, user_email);
+	if (user_name != NULL)
+		cg_window_set_author (cg_plugin->window, user_name);
+
+	if (user_email != NULL)
+		cg_window_set_email (cg_plugin->window, user_email);
 
 	g_free(user_name);
 	g_free(user_email);
