@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     disassemble.c
     Copyright (C) 2006 Sebastien Granjoux
@@ -24,6 +25,8 @@
 /*#define DEBUG*/
 #include <libanjuta/anjuta-debug.h>
 
+#include <libanjuta/interfaces/ianjuta-markable.h>
+
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -42,6 +45,7 @@
 
 enum {DMA_DISASSEMBLY_BUFFER_BLOCK_SIZE = 256,
 	DMA_DISASSEMBLY_SKIP_BEGINNING_LINE = 4,
+	DMA_DISASSEMBLY_TAB_LENGTH = 4,
 	DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH = 8,
 	DMA_DISASSEMBLY_PAGE_DISTANCE = 4 * 60,
 	DMA_DISASSEMBLY_VALID_ADDRESS = 0,
@@ -66,7 +70,7 @@ typedef struct _DmaDisassemblyViewClass DmaDisassemblyViewClass;
 struct _DmaDisassemble
 {
 	IAnjutaDebugger *debugger;
-	AnjutaPlugin *plugin;
+	DebugManagerPlugin *plugin;
 	GtkWidget *window;
 	GtkWidget *menu;
 	DmaSparseBuffer* buffer;
@@ -99,7 +103,7 @@ struct _DmaDisassemblyBufferNode
 	DmaSparseBufferNode parent;
 	guint size;
 	
-	DmaDisassemblyLine data[0];
+	DmaDisassemblyLine data[];
 };
 
 #define DMA_DISASSEMBLY_BUFFER_TYPE              (dma_disassembly_buffer_get_type ())
@@ -146,7 +150,7 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	gint line = -1;
 	DmaDisassemblyBufferNode *node = (DmaDisassemblyBufferNode *)iter->node;
 	
-	DEBUG_PRINT("iter_refresh iter->node %p (%x, %d), base %x offset %d", iter->node, iter->node != NULL ? iter->node->lower : 0, iter->node != NULL ? ((DmaDisassemblyBufferNode *)iter->node)->size : 0, iter->base, iter->offset);
+	DEBUG_PRINT("iter_refresh iter->node %p (%x, %d), base %lx offset %ld", iter->node, iter->node != NULL ? iter->node->lower : 0, iter->node != NULL ? ((DmaDisassemblyBufferNode *)iter->node)->size : 0, iter->base, iter->offset);
 	if (iter->node != NULL)
 	{
 		/* Find line corresponding to base */
@@ -215,7 +219,7 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	}
 
 	/* Try to reduce offset */
-	DEBUG_PRINT("iter_refresh offset iter->node %p, base %x offset %d line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
+	DEBUG_PRINT("iter_refresh offset iter->node %p, base %lx offset %ld line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
 	if (line != -1)
 	{
 		if (iter->offset > 0)
@@ -226,23 +230,27 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 			for (;;)
 			{
 				gint len = node->size - line;
-				
+
 				if (up < len)
 				{
 					iter->node = (DmaSparseBufferNode *)node;
 					iter->line = line + up;
 					iter->base = node->data[iter->line].address;
 					iter->offset = 0;
-					break;
+					
+					return TRUE;
 				}
 
 				if (iter->node->upper == dma_sparse_buffer_get_upper (iter->buffer))
 				{
+					gboolean move = iter->line != node->size - 1;
+					
 					iter->node = (DmaSparseBufferNode *)node;
 					iter->line = node->size - 1;
 					iter->base = node->data[iter->line].address;
 					iter->offset = 0;
-					break;
+					
+					return move;
 				}
 
 				up -= len;
@@ -282,16 +290,20 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 					iter->line = line - down;
 					iter->base = node->data[iter->line].address;
 					iter->offset = 0;
-					break;
+					
+					return TRUE;
 				}
 
 				if (iter->node->lower == dma_sparse_buffer_get_lower (iter->buffer))
 				{
+					gboolean move = iter->line != 0;
+					
 					iter->node = (DmaSparseBufferNode *)node;
 					iter->line = 0;
 					iter->base = node->data[0].address;
 					iter->offset = 0;
-					break;
+					
+					return move;
 				}
 
 				down -= len;
@@ -317,58 +329,56 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	}
 	
 	/* Round offset */
-	DEBUG_PRINT("iter_refresh round iter->node %p, base %x offset %d line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
+	DEBUG_PRINT("iter_refresh round iter->node %p, base %lx offset %ld line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
 	if (iter->offset < 0)
 	{
 		gulong address;
+		gboolean move = TRUE;
 					
 		address = iter->offset + iter->base;
 		if ((address < dma_sparse_buffer_get_lower (iter->buffer)) || (address > iter->base))
 		{
 			address = dma_sparse_buffer_get_lower (iter->buffer);
+			move = FALSE;
 		}
 		address -= address % DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH;
 		iter->offset = address - iter->base;
+		
+		return move;
 	}		
 	else if ((iter->offset > 0) || (iter->line == DMA_DISASSEMBLY_UNKNOWN_ADDRESS))
 	{
 		gulong address;
+		gboolean move = TRUE;
 					
 		address = iter->offset + iter->base;
 		if ((address > dma_sparse_buffer_get_upper (iter->buffer)) || (address < iter->base))
 		{
 			address = dma_sparse_buffer_get_upper (iter->buffer);
+			move = FALSE;
 		}
 		address -= address % DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH;
 		iter->offset = address - iter->base;
+		
+		return move;
 	}
 	
-	/* TODO: return FALSE if iterator reach the lower or upper limit */
+	/* return FALSE if iterator reach the lower or upper limit */
 	return TRUE;
 }
 
 static gboolean
 dma_disassembly_iter_backward_line (DmaSparseIter *iter)
 {
-	gulong old_base = iter->base;
-	gulong old_offset = iter->offset;
-	
 	iter->offset -= DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH;
-	dma_disassembly_iter_refresh (iter);
-	
-	return (old_base != iter->base) || (old_offset != iter->offset);
+	return dma_disassembly_iter_refresh (iter);
 }
 
 static gboolean
 dma_disassembly_iter_forward_line (DmaSparseIter *iter)
 {
-	gulong old_base = iter->base;
-	gulong old_offset = iter->offset;
-
 	iter->offset += DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH;
-	dma_disassembly_iter_refresh (iter);
-	
-	return (old_base != iter->base) || (old_offset != iter->offset);
+	return dma_disassembly_iter_refresh (iter);
 }
 
 
@@ -385,7 +395,6 @@ on_disassemble (const IAnjutaDebuggerDisassembly *block, DmaSparseBufferTranspor
 	DmaDisassemblyBufferNode *node;
 	DmaDisassemblyBuffer *buffer = (DmaDisassemblyBuffer *)trans->buffer;
 	DmaSparseBufferNode *next;
-	guint size;
 	guint i;
 	char *dst;
 	
@@ -436,37 +445,64 @@ on_disassemble (const IAnjutaDebuggerDisassembly *block, DmaSparseBufferTranspor
 	}
 	else
 	{
+		guint size = 0;
+		guint line = 0;
+		
 		DEBUG_PRINT ("on disassemble size %d", block->size);
 		/* Compute size of all data */
-		size = 0;
 		/* use size -1 because last block has no data (NULL) */
 		for (i = trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 0 : 4; i < block->size - 1; i++)
 		{	
-			size += strlen(block->data[i].text) + 1;
+			if (block->data[i].label)
+			{
+				size += strlen(block->data[i].label) + 2;
+				line++;
+			}
+			size += strlen(block->data[i].text) + 1 + DMA_DISASSEMBLY_TAB_LENGTH;
+			line++;
 		}
 	
-		node = (DmaDisassemblyBufferNode *)g_malloc0 (sizeof(DmaDisassemblyBufferNode) + sizeof(DmaDisassemblyLine) * block->size + size);
+		node = (DmaDisassemblyBufferNode *)g_malloc0 (sizeof(DmaDisassemblyBufferNode) + sizeof(DmaDisassemblyLine) * line + size);
 	
 		/* Copy all data */
-		size = 0;
-		dst = (gchar *)&(node->data[block->size]);
+		dst = (gchar *)&(node->data[line]);
+		line = 0;
 		for (i = trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 0 : DMA_DISASSEMBLY_SKIP_BEGINNING_LINE; i < block->size - 1; i++)
 		{
 			gsize len;
 
 			if ((next != NULL) && (block->data[i].address == next->lower)) break;
-			
+
+			/* Add label if exist */
+			if (block->data[i].label != NULL)
+			{
+				len = strlen(block->data[i].label);
+				
+				node->data[line].address = block->data[i].address;
+				node->data[line].text = dst;
+				
+				memcpy(dst, block->data[i].label, len);
+				dst[len] = ':';
+				dst[len + 1] = '\0';
+				
+				dst += len + 2;
+				line++;
+			}
+				
+			/* Add disassembled instruction */
 			len = strlen(block->data[i].text) + 1;
 
-			node->data[i - (trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 0 : DMA_DISASSEMBLY_SKIP_BEGINNING_LINE)].address = block->data[i].address;
-			node->data[i - (trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 0 : DMA_DISASSEMBLY_SKIP_BEGINNING_LINE)].text = dst;
+			node->data[line].address = block->data[i].address;
+			node->data[line].text = dst;
 		
-			memcpy (dst, block->data[i].text, len);
-			dst += len;
+			memset (dst, ' ', DMA_DISASSEMBLY_TAB_LENGTH);
+			memcpy (dst + DMA_DISASSEMBLY_TAB_LENGTH, block->data[i].text, len);
+			dst += len + DMA_DISASSEMBLY_TAB_LENGTH;
+			line++;
 		}
 
 		/* fill last block */
-		node->size = i - (trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 1 : DMA_DISASSEMBLY_SKIP_BEGINNING_LINE + 1 );
+		node->size = line;
 		node->parent.lower = node->data[0].address;
 		node->parent.upper = block->data[i].address - 1;
 	
@@ -497,7 +533,7 @@ dma_disassembly_buffer_insert_line (DmaSparseIter *iter, GtkTextIter *dst)
 				gulong end_adr;
 				gint margin;
 			
-				DEBUG_PRINT("no data at address %x + %d", iter->base, iter->offset);
+				DEBUG_PRINT("no data at address %lx + %ld", iter->base, iter->offset);
 				
 				/* If following line is define, get a block stopping here */
 				dma_sparse_iter_copy (&end, iter);
@@ -717,6 +753,19 @@ on_disassembly_buffer_changed (DmaDisassemblyBuffer *buffer, DmaSparseView *view
 	dma_sparse_view_refresh (view);
 }
 
+static void
+on_breakpoint_changed (DmaDisassemble *self, IAnjutaDebuggerBreakpoint *bp)
+{
+	g_return_if_fail (bp != NULL);
+	
+	dma_disassemble_unmark (self, bp->address, IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
+	dma_disassemble_unmark (self, bp->address, IANJUTA_MARKABLE_BREAKPOINT_ENABLED);
+	if (bp->type != IANJUTA_DEBUGGER_BREAK_REMOVED)
+	{
+		dma_disassemble_mark (self, bp->address, bp->enable ? IANJUTA_MARKABLE_BREAKPOINT_ENABLED : IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
+	}
+}
+
 static GtkWidget*
 create_disassemble_gui (DmaDisassemble *self)
 {
@@ -733,7 +782,7 @@ create_disassemble_gui (DmaDisassemble *self)
 									 GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (self->window), GTK_WIDGET (dataview));
 	
-	anjuta_shell_add_widget (self->plugin->shell,
+	anjuta_shell_add_widget (ANJUTA_PLUGIN (self->plugin)->shell,
 							 self->window,
                              "AnjutaDebuggerDisassemble", _("Disassembly"),
                              NULL, ANJUTA_SHELL_PLACEMENT_LEFT,
@@ -751,6 +800,8 @@ on_debugger_started (DmaDisassemble *self)
 	self->buffer = DMA_SPARSE_BUFFER (dma_disassembly_buffer_new (self->debugger, 0x00000000U,0xFFFFFFFFU));
 
 	self->view = DMA_SPARSE_VIEW (create_disassemble_gui (self));
+
+	g_signal_connect_swapped (self->plugin, "breakpoint-changed", G_CALLBACK (on_breakpoint_changed), self);
 	
 	g_signal_connect (G_OBJECT (self->buffer), 
 			"changed", 
@@ -761,6 +812,8 @@ on_debugger_started (DmaDisassemble *self)
 static void
 destroy_disassemble_gui (DmaDisassemble *self)
 {
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_breakpoint_changed), self);
+
 	/* Destroy menu */
 	if (self->menu != NULL)
 	{
@@ -789,6 +842,45 @@ on_debugger_stopped (DmaDisassemble *self)
 	destroy_disassemble_gui (self);
 }
 
+/* Public functions
+ *---------------------------------------------------------------------------*/
+
+void
+dma_disassemble_mark (DmaDisassemble *self, guint address, gint marker)
+{
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (self->view != NULL);
+	
+	dma_sparse_view_mark (self->view, address, marker);
+}
+
+void
+dma_disassemble_unmark (DmaDisassemble *self, guint address, gint marker)
+{
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (self->view != NULL);
+	
+	dma_sparse_view_unmark (self->view, address, marker);
+}
+
+void
+dma_disassemble_clear_all_mark (DmaDisassemble *self, gint marker)
+{
+	g_return_if_fail (self != NULL);
+	/* Accept to clear mark without a view, used at initialization */
+	if (self->view != NULL)
+		dma_sparse_view_delete_all_markers (self->view, marker);
+}
+
+void
+dma_disassemble_goto_address (DmaDisassemble *self, guint address)
+{
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (self->view != NULL);
+	
+	dma_sparse_view_goto (self->view, address);
+}
+
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
@@ -801,13 +893,13 @@ dma_disassemble_new(AnjutaPlugin *plugin, IAnjutaDebugger *debugger)
 	
 	self->debugger = debugger;
 	if (debugger != NULL) g_object_ref (debugger);
-	self->plugin = plugin;
+	self->plugin = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
 	self->buffer = NULL;
 	self->view = NULL;
 
 	g_signal_connect_swapped (self->debugger, "debugger-started", G_CALLBACK (on_debugger_started), self);
 	g_signal_connect_swapped (self->debugger, "debugger-stopped", G_CALLBACK (on_debugger_stopped), self);
-
+	
 	return self;
 }
 
