@@ -21,6 +21,7 @@
 #include <config.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-status.h>
@@ -686,7 +687,7 @@ open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 	AnjutaPluginDescription *desc;
 	AnjutaPluginManager *plugin_manager;
 	
-	idx = (gint) g_object_get_data (G_OBJECT (menuitem), "index");
+	idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "index"));
 	desc = (AnjutaPluginDescription*) g_object_get_data (G_OBJECT (menuitem),
 														 "desc");
 	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
@@ -865,7 +866,7 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 												  "Location", &name);
 		}
 		menuitem = gtk_menu_item_new_with_label (name);
-		g_object_set_data (G_OBJECT (menuitem), "index", (gpointer)(idx));
+		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
 		g_object_set_data (G_OBJECT (menuitem), "desc", (gpointer)(desc));
 		g_signal_connect (G_OBJECT (menuitem), "activate",
 						  G_CALLBACK (callback), callback_data);
@@ -890,7 +891,7 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 	{
 		mime_app = (GnomeVFSMimeApplication *)(node->data);
 		menuitem = gtk_menu_item_new_with_label (mime_app->name);
-		g_object_set_data (G_OBJECT (menuitem), "index", (gpointer)(idx));
+		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
 		g_signal_connect (G_OBJECT (menuitem), "activate",
 						  G_CALLBACK (callback), callback_data);
 		gtk_menu_append (menu, menuitem);
@@ -1289,6 +1290,15 @@ iloader_load (IAnjutaFileLoader *loader, const gchar *uri,
 	
 	g_return_val_if_fail (uri != NULL, NULL);
 	vfs_uri = gnome_vfs_uri_new (uri);
+	
+	if (!gnome_vfs_uri_exists (vfs_uri))
+	{
+		launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader),
+									uri, GNOME_VFS_ERROR_NOT_FOUND);
+		gnome_vfs_uri_unref (vfs_uri);
+		return NULL;
+	}
+	
 	new_uri = gnome_vfs_uri_to_string (vfs_uri,
 									   GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
 	mime_type = anjuta_util_get_uri_mime_type (new_uri);
