@@ -1,7 +1,7 @@
 
 #include <libanjuta/anjuta-plugin.h>
 
-extern GType gtodo_plugin_get_type (GluePlugin *plugin);
+extern GType gtodo_plugin_get_type (AnjutaGluePlugin *plugin);
 #define ANJUTA_TYPE_PLUGIN_GTODO         (gtodo_plugin_get_type (NULL))
 #define ANJUTA_PLUGIN_GTODO(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_TYPE_PLUGIN_GTODO, GTodoPlugin))
 #define ANJUTA_PLUGIN_GTODO_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), ANJUTA_TYPE_PLUGIN_GTODO, GTodoPluginClass))
