@@ -28,6 +28,7 @@ struct _AnjutaMsgmanPriv
 {
 	AnjutaPreferences *preferences;
 	GtkWidget* popup_menu;
+	GtkWidget* tab_popup;
 	MessageView *current_view;
 	GList *views;
 };
@@ -53,6 +54,53 @@ on_msgman_close_page (GtkButton* button,
 	anjuta_msgman_remove_view (msgman, view);
 }
 
+static void 
+on_msgman_close_all(GtkMenuItem* item, AnjutaMsgman* msgman)
+{
+	
+	anjuta_msgman_remove_all_views(msgman);
+}
+
+static GtkWidget*
+create_tab_popup_menu(AnjutaMsgman* msgman)
+{
+	GtkWidget* menu;
+	GtkWidget* item_close_all;
+	
+	menu = gtk_menu_new();
+	
+	item_close_all = gtk_menu_item_new_with_label(_("Close all message tabs"));
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item_close_all);
+	g_signal_connect(G_OBJECT(item_close_all), "activate", G_CALLBACK(on_msgman_close_all),
+					 msgman);
+	gtk_widget_show_all(menu);
+	
+	gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(msgman), NULL);
+	return menu;
+}
+
+static void
+on_msgman_popup_menu(GtkWidget* widget, AnjutaMsgman* msgman)
+{
+	gtk_menu_popup(GTK_MENU(msgman->priv->tab_popup),
+				   NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
+}
+
+static gboolean
+on_tab_button_press_event(GtkWidget* widget, GdkEventButton* event, AnjutaMsgman* msgman)
+{
+	if (event->type == GDK_BUTTON_PRESS)
+	{
+		if (event->button == 3)
+		{
+				gtk_menu_popup(GTK_MENU(msgman->priv->tab_popup), NULL, NULL, NULL, NULL, 
+							   event->button, event->time);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static AnjutaMsgmanPage *
 anjuta_msgman_page_new (GtkWidget * view, const gchar * name,
 			const gchar * pixmap, AnjutaMsgman * msgman)
@@ -67,6 +115,7 @@ anjuta_msgman_page_new (GtkWidget * view, const gchar * name,
 	page->widget = GTK_WIDGET (view);
 	
 	page->label = gtk_label_new (name);
+	gtk_misc_set_alignment (GTK_MISC(page->label), 0.0, 0.5);
 	page->box = gtk_hbox_new (FALSE, 0);
 	gtk_box_set_spacing (GTK_BOX (page->box), 5);
 	if (pixmap  && strlen(pixmap))
@@ -74,7 +123,7 @@ anjuta_msgman_page_new (GtkWidget * view, const gchar * name,
 		GtkStockItem unused;
 		if (gtk_stock_lookup(pixmap, &unused))
 		{
-			page->pixmap = gtk_image_new_from_stock(pixmap, GTK_ICON_SIZE_SMALL_TOOLBAR);
+			page->pixmap = gtk_image_new_from_stock(pixmap, GTK_ICON_SIZE_MENU);
 		}
 		else
 		{
@@ -101,11 +150,9 @@ anjuta_msgman_page_new (GtkWidget * view, const gchar * name,
 	gtk_box_pack_start (GTK_BOX (page->box), page->button, FALSE, FALSE, 0);
 	
 	g_object_set_data (G_OBJECT (page->button), "message_view", page->widget);
-	gtk_signal_connect (GTK_OBJECT (page->button), "clicked",
-						GTK_SIGNAL_FUNC(on_msgman_close_page),
+	g_signal_connect (GTK_OBJECT (page->button), "clicked",
+						G_CALLBACK(on_msgman_close_page),
 						msgman);
-	
-	gtk_widget_show_all (page->box);
 	return page;
 }
 
@@ -123,9 +170,8 @@ on_notebook_switch_page (GtkNotebook * notebook,
 	g_return_if_fail (notebook != NULL);
 	g_return_if_fail (page != NULL);
 	g_return_if_fail (msgman != NULL);
-
-	msgman->priv->current_view =
-		MESSAGE_VIEW (gtk_notebook_get_nth_page (notebook, page_num));
+	
+	anjuta_msgman_set_current_view(msgman, NULL);
 }
 
 static gpointer parent_class;
@@ -145,6 +191,7 @@ static void
 anjuta_msgman_finalize (GObject *obj)
 {
 	AnjutaMsgman *msgman = ANJUTA_MSGMAN (obj);
+	gtk_widget_destroy(msgman->priv->tab_popup);
 	if (msgman->priv)
 	{
 		g_free (msgman->priv);
@@ -162,13 +209,35 @@ anjuta_msgman_instance_init (AnjutaMsgman * msgman)
 	msgman->priv = g_new0(AnjutaMsgmanPriv, 1);
 	msgman->priv->views = NULL;
 	msgman->priv->current_view = NULL;
+	msgman->priv->tab_popup = create_tab_popup_menu(msgman);
+	g_signal_connect(GTK_OBJECT(msgman), "popup-menu", 
+                       G_CALLBACK(on_msgman_popup_menu), msgman);
+    g_signal_connect(GTK_OBJECT(msgman), "button-press-event", 
+                       G_CALLBACK(on_tab_button_press_event), msgman);
 }
 
 static void
 anjuta_msgman_class_init (AnjutaMsgmanClass * klass)
 {
+	static gboolean initialized = FALSE;
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	
+	if (!initialized) {
+		/* Signal */
+		g_signal_new ("view-changed",
+			ANJUTA_TYPE_MSGMAN,
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (AnjutaMsgman, view_changed),
+			NULL, NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE,
+			0,
+			NULL);
+
+
+		initialized = TRUE;
+	}
+	
 	parent_class = g_type_class_peek_parent (klass);
 	gobject_class->finalize = anjuta_msgman_finalize;
 	gobject_class->dispose = anjuta_msgman_dispose;
@@ -287,9 +356,10 @@ anjuta_msgman_append_view (AnjutaMsgman * msgman, GtkWidget *mv,
 	g_return_if_fail (mv != NULL);
 	g_return_if_fail (name != NULL);
 
-	gtk_widget_show (mv);
+	gtk_widget_show_all (mv);
 	page = anjuta_msgman_page_new (mv, name, pixmap, msgman);
-
+	gtk_widget_show_all (page->box);
+	
 	g_signal_handlers_block_by_func (GTK_OBJECT (msgman),
 									 GTK_SIGNAL_FUNC
 									 (on_notebook_switch_page), msgman);
@@ -305,6 +375,7 @@ anjuta_msgman_append_view (AnjutaMsgman * msgman, GtkWidget *mv,
 	g_signal_handlers_unblock_by_func (GTK_OBJECT (msgman),
 									   GTK_SIGNAL_FUNC
 									   (on_notebook_switch_page), msgman);
+	g_signal_emit_by_name(G_OBJECT(msgman), "view_changed");
 }
 
 MessageView *
@@ -335,6 +406,7 @@ anjuta_msgman_remove_view (AnjutaMsgman * msgman, MessageView *passed_view)
 
 	g_return_if_fail (view != NULL);
 	gtk_widget_destroy (GTK_WIDGET (view));
+	anjuta_msgman_set_current_view(msgman, NULL);
 }
 
 void
@@ -405,17 +477,31 @@ void
 anjuta_msgman_set_current_view (AnjutaMsgman * msgman, MessageView * mv)
 {
 	g_return_if_fail (msgman != NULL);
-//	g_return_if_fail (mv != NULL);
-	if (mv == NULL) return;
-
 	AnjutaMsgmanPage *page;
 	gint page_num;
 
-	page = anjuta_msgman_page_from_widget (msgman, mv);
-	page_num =
-		gtk_notebook_page_num (GTK_NOTEBOOK (msgman),
-				       GTK_WIDGET (mv));
-	gtk_notebook_set_page (GTK_NOTEBOOK (msgman), page_num);
+	if (mv == NULL)
+	{
+		if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(msgman)) == 0)
+		{
+			msgman->priv->current_view = NULL;
+		}
+		else
+		{
+			msgman->priv->current_view = 
+				MESSAGE_VIEW(gtk_notebook_get_nth_page(GTK_NOTEBOOK(msgman),
+													   gtk_notebook_get_current_page(GTK_NOTEBOOK(msgman))));
+		}
+	}
+	else
+	{
+		page = anjuta_msgman_page_from_widget (msgman, mv);
+		page_num =
+			gtk_notebook_page_num (GTK_NOTEBOOK (msgman),
+					       GTK_WIDGET (mv));
+		gtk_notebook_set_page (GTK_NOTEBOOK (msgman), page_num);
+	}
+	g_signal_emit_by_name(G_OBJECT(msgman), "view_changed");
 }
 
 GList *
