@@ -38,6 +38,7 @@ struct _MessageViewPrivate
 	
 	/* Properties */
 	gchar *label;
+	gchar *pixmap;
 	gboolean highlite;
 	
 	GdkRectangle tooltip_rect;
@@ -62,6 +63,7 @@ enum
 	COLUMN_COLOR = 0,
 	COLUMN_SUMMARY,
 	COLUMN_MESSAGE,
+	COLUMN_PIXBUF,
 	N_COLUMNS
 };
 
@@ -69,6 +71,7 @@ enum
 {
 	MV_PROP_ID = 0,
 	MV_PROP_LABEL,
+	MV_PROP_PIXMAP,
 	MV_PROP_HIGHLITE
 };
 
@@ -132,6 +135,38 @@ message_free (Message *message)
 	g_free (message);
 }
 
+static gboolean
+message_serialize (Message *message, AnjutaSerializer *serializer)
+{
+	if (!anjuta_serializer_write_int (serializer, "type",
+									  message->type))
+		return FALSE;
+	if (!anjuta_serializer_write_string (serializer, "summary",
+										 message->summary))
+		return FALSE;
+	if (!anjuta_serializer_write_string (serializer, "details",
+										 message->details))
+		return FALSE;
+	return TRUE;
+}
+
+static gboolean
+message_deserialize (Message *message, AnjutaSerializer *serializer)
+{
+	gint type;
+	if (!anjuta_serializer_read_int (serializer, "type",
+									 &type))
+		return FALSE;
+	message->type = type;
+	if (!anjuta_serializer_read_string (serializer, "summary",
+										&message->summary, TRUE))
+		return FALSE;
+	if (!anjuta_serializer_read_string (serializer, "details",
+										&message->details, TRUE))
+		return FALSE;
+	return TRUE;
+}
+
 static GType
 message_get_type ()
 {
@@ -468,7 +503,7 @@ on_adjustment_changed (GtkAdjustment* adj, gpointer data)
 static void
 on_adjustment_value_changed (GtkAdjustment* adj, gpointer data)
 {
-	MessageView *self = (MessageView *) data;
+	MessageView *self = MESSAGE_VIEW (data);
 	if (adj->value > (adj->upper - adj->page_size) - 0.1)
 	{
 		if (!self->privat->adj_chgd_hdlr)
@@ -505,6 +540,12 @@ message_view_set_property (GObject * object,
 		self->privat->label = g_value_dup_string (value);
 		break;
 	}
+	case MV_PROP_PIXMAP:
+	{
+		g_free (self->privat->pixmap);
+		self->privat->pixmap = g_value_dup_string (value);
+		break;
+	}
 	case MV_PROP_HIGHLITE:
 	{
 		self->privat->highlite = g_value_get_boolean (value);
@@ -512,7 +553,7 @@ message_view_set_property (GObject * object,
 	}
 	default:
 	{
-		g_assert ("Unknown property");
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
 	}
 	}
@@ -535,6 +576,11 @@ message_view_get_property (GObject * object,
 			g_value_set_string (value, self->privat->label);
 			break;
 		}
+		case MV_PROP_PIXMAP:
+		{
+			g_value_set_string (value, self->privat->pixmap);
+			break;
+		}
 		case MV_PROP_HIGHLITE:
 		{
 			g_value_set_boolean (value, self->privat->highlite);
@@ -542,7 +588,7 @@ message_view_get_property (GObject * object,
 		}
 		default:
 		{
-			g_assert ("Unknown property");
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 			break;
 		}
 	}
@@ -577,10 +623,9 @@ static void
 message_view_finalize (GObject *obj)
 {
 	MessageView *mview = MESSAGE_VIEW (obj);
-	if (mview->privat->line_buffer)
-		g_free (mview->privat->line_buffer);
-	if (mview->privat->label)
-		g_free (mview->privat->label);
+	g_free (mview->privat->line_buffer);
+	g_free (mview->privat->label);
+	g_free (mview->privat->pixmap);
 	g_free (mview->privat);
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT(obj)));
 }
@@ -590,7 +635,9 @@ message_view_instance_init (MessageView * self)
 {
 	GtkWidget *scrolled_win;
 	GtkCellRenderer *renderer;
+	GtkCellRenderer *renderer_pixbuf;
 	GtkTreeViewColumn *column;
+	GtkTreeViewColumn *column_pixbuf;
 	GtkTreeSelection *select;
 	GtkListStore *model;
 	GtkAdjustment* adj;
@@ -603,17 +650,27 @@ message_view_instance_init (MessageView * self)
 
 	/* Create the tree widget */
 	model = gtk_list_store_new (N_COLUMNS, GDK_TYPE_COLOR,
-								G_TYPE_STRING,
-								MESSAGE_TYPE);
+								G_TYPE_STRING, MESSAGE_TYPE,  G_TYPE_STRING);
 	self->privat->tree_view =
 		gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	gtk_widget_show (self->privat->tree_view);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW
 									   (self->privat->tree_view), FALSE);
 
-	/* Creat columns to hold text and color of a line, this
+	/* Create pixbuf column */
+	renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
+	column_pixbuf = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (column_pixbuf, _("Icon"));
+	gtk_tree_view_column_pack_start (column_pixbuf, renderer_pixbuf, TRUE);
+	gtk_tree_view_column_add_attribute
+		(column_pixbuf, renderer_pixbuf, "stock-id", COLUMN_PIXBUF);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (self->privat->tree_view),
+								 column_pixbuf);
+	/* Create columns to hold text and color of a line, this
 	 * columns are invisible of course. */
 	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "yalign", 0.0, "wrap-mode", PANGO_WRAP_WORD,
+				  "wrap-width", 1000, NULL);
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_title (column, _("Messages"));
@@ -661,6 +718,7 @@ static void
 message_view_class_init (MessageViewClass * klass)
 {
 	GParamSpec *message_view_spec_label;
+	GParamSpec *message_view_spec_pixmap;
 	GParamSpec *message_view_spec_highlite;
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	
@@ -680,6 +738,16 @@ message_view_class_init (MessageViewClass * klass)
 					 MV_PROP_LABEL,
 					 message_view_spec_label);
 
+	message_view_spec_pixmap = g_param_spec_string ("pixmap",
+						       "Pixmap of the view",
+						       "Used to decorate the view tab,"
+						       "translateable",
+						       "no label",
+						       G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class,
+					 MV_PROP_PIXMAP,
+					 message_view_spec_pixmap);
+
 	message_view_spec_highlite = g_param_spec_boolean ("highlite",
 							   "Highlite build messages",
 							   "If TRUE, specify colors",
@@ -688,7 +756,6 @@ message_view_class_init (MessageViewClass * klass)
 	g_object_class_install_property (gobject_class,
 					 MV_PROP_HIGHLITE,
 					 message_view_spec_highlite);
-
 }
 
 /* Returns a new message-view instance */
@@ -702,6 +769,86 @@ message_view_new (AnjutaPreferences* prefs, GtkWidget* popup_menu)
 	return GTK_WIDGET(mv);
 }
 
+gboolean
+message_view_serialize (MessageView *view, AnjutaSerializer *serializer)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	if (!anjuta_serializer_write_string (serializer, "label",
+										 view->privat->label))
+		return FALSE;
+	if (!anjuta_serializer_write_string (serializer, "pixmap",
+										 view->privat->pixmap))
+		return FALSE;
+	if (!anjuta_serializer_write_int (serializer, "highlite",
+									  view->privat->highlite))
+		return FALSE;
+	
+	/* Serialize individual messages */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->privat->tree_view));
+	
+	if (!anjuta_serializer_write_int (serializer, "messages",
+									  gtk_tree_model_iter_n_children (model, NULL)))
+		return FALSE;
+
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			Message *message;
+			gtk_tree_model_get (model, &iter, COLUMN_MESSAGE, &message, -1);
+			if (message)
+			{
+				if (!message_serialize (message, serializer))
+					return FALSE;
+			}
+		}
+		while (gtk_tree_model_iter_next (model, &iter));
+	}
+	return TRUE;
+}
+
+gboolean
+message_view_deserialize (MessageView *view, AnjutaSerializer *serializer)
+{
+	GtkTreeModel *model;
+	gint messages, i;
+	
+	if (!anjuta_serializer_read_string (serializer, "label",
+										&view->privat->label, TRUE))
+		return FALSE;
+	if (!anjuta_serializer_read_string (serializer, "pixmap",
+										&view->privat->pixmap, TRUE))
+		return FALSE;
+	if (!anjuta_serializer_read_int (serializer, "highlite",
+									 &view->privat->highlite))
+		return FALSE;
+	
+	/* Create individual messages */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->privat->tree_view));
+	gtk_list_store_clear (GTK_LIST_STORE (model));
+	
+	if (!anjuta_serializer_read_int (serializer, "messages", &messages))
+		return FALSE;
+	
+	for (i = 0; i < messages; i++)
+	{
+		Message *message;
+
+		message = message_new (0, NULL, NULL);
+		if (!message_deserialize (message, serializer))
+		{
+			message_free (message);
+			return FALSE;
+		}
+		ianjuta_message_view_append (IANJUTA_MESSAGE_VIEW (view), message->type,
+									 message->summary, message->details, NULL);
+		message_free (message);
+	}
+	return TRUE;
+}
+
 void message_view_next(MessageView* view)
 {
 	GtkTreeIter iter;
@@ -734,9 +881,9 @@ void message_view_next(MessageView* view)
 			{
 				GtkTreePath *path;
 				path = gtk_tree_model_get_path (model, &iter);
-				gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW
+				gtk_tree_view_set_cursor (GTK_TREE_VIEW
 											  (view->privat->tree_view),
-											  path, NULL, FALSE, 0, 0);
+											  path, NULL, FALSE);
 				gtk_tree_path_free (path);
 				g_signal_emit_by_name (G_OBJECT (view), "message_clicked", 
 									   message);
@@ -894,6 +1041,7 @@ pref_change_color (MessageView *mview, IAnjutaMessageViewType type,
 		}
 		success = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
 	}
+	g_free(color);
 }
 
 static void
@@ -1010,6 +1158,7 @@ imessage_view_append (IAnjutaMessageView *message_view,
 	gboolean highlite;
 	gchar *utf8_msg;
 	gchar *escaped_str;
+	gchar* stock_id;
 	
 	MessageView *view;
 	Message *message;
@@ -1029,14 +1178,18 @@ imessage_view_append (IAnjutaMessageView *message_view,
 			case IANJUTA_MESSAGE_VIEW_TYPE_INFO:
 				color = anjuta_util_convert_color(view->privat->prefs,
 									  "messages.color.info");
+				stock_id = GTK_STOCK_INFO;
 				break;
 			case IANJUTA_MESSAGE_VIEW_TYPE_WARNING:
 				color = anjuta_util_convert_color(view->privat->prefs,
 									  "messages.color.warning");
+				/* FIXME: There is no GTK_STOCK_WARNING which would fit better here */
+				stock_id = GTK_STOCK_DIALOG_WARNING;
 				break;
 			case IANJUTA_MESSAGE_VIEW_TYPE_ERROR:
 				color = anjuta_util_convert_color(view->privat->prefs,
 									  "messages.color.error");
+				stock_id = GTK_STOCK_STOP;
 				break;
 			default:
 				color = NULL;
@@ -1069,6 +1222,7 @@ imessage_view_append (IAnjutaMessageView *message_view,
 							COLUMN_COLOR, color,
 							COLUMN_SUMMARY, escaped_str,
 							COLUMN_MESSAGE, message,
+							COLUMN_PIXBUF, stock_id,
 							-1);
 	
 		/* Can we free the color when it's in the tree_view? */
