@@ -16,6 +16,7 @@ char *s = N_("Create glade interface file");
 char *s = N_("Destination:");
 char *s = N_("Display description of the plugin");
 char *s = N_("Display title of the plugin");
+char *s = N_("Django Project information");
 char *s = N_("Email address:");
 char *s = N_("GCJ needs to know which class contains the main() function");
 char *s = N_("General Project Information");
@@ -31,6 +32,7 @@ char *s = N_("Lesser General Public License(LGPL)");
 char *s = N_("License");
 char *s = N_("Main Class:");
 char *s = N_("Main class");
+char *s = N_("Mininum SDL version required");
 char *s = N_("Name of the value to watch");
 char *s = N_("No license");
 char *s = N_("Options for project build system");
@@ -42,9 +44,23 @@ char *s = N_("Plugin class name");
 char *s = N_("Plugin has menus or/and toolbars");
 char *s = N_("Plugin interfaces to implement");
 char *s = N_("Project Name:");
+char *s = N_("Project directory, output file etc.");
+char *s = N_("Project directory:");
 char *s = N_("Project name must not contain spaces, because it will be the name of the project build target (executable, library etc.)");
+char *s = N_("Project name:");
 char *s = N_("Project options");
 char *s = N_("Require Package:");
+char *s = N_("Require SDL version:");
+char *s = N_("Require SDL_gfx library");
+char *s = N_("Require SDL_gfx:");
+char *s = N_("Require SDL_image library");
+char *s = N_("Require SDL_image:");
+char *s = N_("Require SDL_mixer library");
+char *s = N_("Require SDL_mixer:");
+char *s = N_("Require SDL_net library");
+char *s = N_("Require SDL_net:");
+char *s = N_("Require SDL_ttf library");
+char *s = N_("Require SDL_ttf:");
 char *s = N_("Select code license");
 char *s = N_("Shell values to watch");
 char *s = N_("Use pkg-config to add library supports from other packages");
