@@ -34,7 +34,7 @@
 #include <gtk/gtk.h>
 
 #include "anjuta-document.h"
-#include "tag-window.h"
+#include "sourceview.h"
 #include <gtksourceview/gtksourceview.h>
 
 G_BEGIN_DECLS
@@ -69,7 +69,7 @@ struct _AnjutaView
  * Class definition
  */
 typedef struct _AnjutaViewClass		AnjutaViewClass;
- 
+
 struct _AnjutaViewClass
 {
 	GtkSourceViewClass parent_class;
@@ -84,7 +84,7 @@ struct _AnjutaViewClass
  */
 GtkType		 anjuta_view_get_type     	(void) G_GNUC_CONST;
 
-GtkWidget	*anjuta_view_new			(AnjutaDocument   *doc);
+GtkWidget	*anjuta_view_new			(Sourceview   *sv);
 
 void		 anjuta_view_cut_clipboard 	(AnjutaView       *view);
 void		 anjuta_view_copy_clipboard 	(AnjutaView       *view);
@@ -105,8 +105,6 @@ void 		 anjuta_view_set_font		(AnjutaView       *view,
 						 gboolean         def,
 						 const gchar     *font_name);
 
-void 		 anjuta_view_register_completion(AnjutaView* view, TagWindow* tagwin);
-
 G_END_DECLS
 
 #endif /* __ANJUTA_VIEW_H__ */
